@@ -7,6 +7,7 @@ package W3C::SOAP::XSD::Document::ComplexType;
 # $Revision$, $Source$, $Date$
 
 use Moose;
+use warnings;
 use version;
 use Carp;
 use Scalar::Util;
@@ -18,7 +19,7 @@ use W3C::SOAP::Utils qw/split_ns/;
 
 extends 'W3C::SOAP::XSD::Document::Type';
 
-our $VERSION     = version->new('0.0.2');
+our $VERSION     = version->new('0.0.3');
 
 has sequence => (
     is      => 'rw',
@@ -69,7 +70,7 @@ sub _extension {
 
     for my $node (@nodes) {
         my ($ns, $tag) = split_ns($node->getAttribute('base'));
-        my $ns_uri = $self->document->get_ns_uri($ns);
+        my $ns_uri = $self->document->get_ns_uri($ns, $self->node);
 
         return $self->document->get_module_base( $ns_uri ) . "::$tag";
     }
@@ -116,7 +117,7 @@ W3C::SOAP::XSD::Document::ComplexType - <One-line description of module's purpos
 
 =head1 VERSION
 
-This documentation refers to W3C::SOAP::XSD::Document::ComplexType version 0.0.2.
+This documentation refers to W3C::SOAP::XSD::Document::ComplexType version 0.0.3.
 
 
 =head1 SYNOPSIS
