@@ -7,6 +7,7 @@ package W3C::SOAP::XSD::Document::Element;
 # $Revision$, $Source$, $Date$
 
 use Moose;
+use warnings;
 use version;
 use Carp;
 use Scalar::Util;
@@ -18,7 +19,7 @@ use W3C::SOAP::Utils qw/split_ns xml_error/;
 
 extends 'W3C::SOAP::XSD::Document::Type';
 
-our $VERSION     = version->new('0.0.2');
+our $VERSION     = version->new('0.0.3');
 
 has complex_type => (
     is     => 'rw',
@@ -83,8 +84,8 @@ sub _type {
     for my $type (keys %{$simple}) {
         my $node = $simple->{$type}->node;
         my  $type_name = $node->parentNode->getAttribute('name');
-        if ( $type_name && $type_name eq $self->name ) {
-            my @children = $node->findnodes('xs:restriction', $node);
+        if ( $type_name && $self->name && $type_name eq $self->name ) {
+            my @children = $self->document->xpc->findnodes('xsd:restriction', $node);
             last if @children != 1;
 
             my $child = $children[0]->firstChild;
@@ -105,7 +106,8 @@ sub _package {
     my ($self) = @_;
     my $type = $self->type;
     my ($ns, $name) = split_ns($type);
-    my $ns_uri = $name ? $self->document->get_ns_uri($ns) : '';
+    $ns ||= $self->document->target_namespace;
+    my $ns_uri = $name ? $self->document->get_ns_uri($ns, $self->node) : '';
     $name ||= $ns;
 
     if ( $ns_uri eq 'http://www.w3.org/2001/XMLSchema' ) {
@@ -141,16 +143,20 @@ sub module {
 sub type_module {
     my ($self) = @_;
     my ($ns, $type) = split_ns($self->type);
-    my $ns_uri = $self->document->get_ns_uri($ns);
+    $ns ||= $self->document->target_namespace;
+    my $ns_uri = $self->document->get_ns_uri($ns, $self->node);
 
     return $self->simple_type || $self->document->get_module_base( $ns_uri ) . '::' . $type;
 }
 
 sub simple_type {
     my ($self) = @_;
-    $self->document->simple_type;
+    $self->document->simple_type();
     my ($ns, $type) = split_ns($self->type);
-    my $ns_uri = $self->document->get_ns_uri($ns);
+    $ns ||= $self->document->target_namespace;
+    return "xs:$type" if $self->document->ns_map->{$ns} && $self->document->ns_map->{$ns} eq 'http://www.w3.org/2001/XMLSchema';
+
+    my $ns_uri = $self->document->get_ns_uri($ns, $self->node);
     warn "Simple type missing a type for '".$self->type."'\n".xml_error($self->node)."\n"
         if !$ns && $ns_uri ne 'http://www.w3.org/2001/XMLSchema';
 
@@ -173,9 +179,12 @@ sub simple_type {
 
 sub very_simple_type {
     my ($self) = @_;
-    $self->document->simple_type;
+    $self->document->simple_type();
     my ($ns, $type) = split_ns($self->type);
-    my $ns_uri = $self->document->get_ns_uri($ns);
+    $ns ||= $self->document->target_namespace;
+    return "xs:$type" if $self->document->ns_map->{$ns} && $self->document->ns_map->{$ns} eq 'http://www.w3.org/2001/XMLSchema';
+
+    my $ns_uri = $self->document->get_ns_uri($ns, $self->node);
     warn "Simple type missing a type for '".$self->type."'\n".xml_error($self->node)."\n"
         if !$ns && $ns_uri ne 'http://www.w3.org/2001/XMLSchema';
 
@@ -198,7 +207,8 @@ sub very_simple_type {
 sub moosex_type {
     my ($self) = @_;
     my ($ns, $type) = split_ns($self->type);
-    my $ns_uri = $self->document->get_ns_uri($ns);
+    $ns ||= $self->document->target_namespace;
+    my $ns_uri = $self->document->get_ns_uri($ns, $self->node);
     warn "Simple type missing a type for '".$self->type."'\n".xml_error($self->node)."\n"
         if !$ns && $ns_uri ne 'http://www.w3.org/2001/XMLSchema';
 
@@ -226,7 +236,7 @@ sub has_anonymous {
     my $simple = $self->document->simple_type;
     for my $type (keys %{$simple}) {
         my  $type_name = $simple->{$type}->node->parentNode->getAttribute('name');
-        if ( $type_name && $type_name eq $self->name ) {
+        if ( $type_name && $self->name && $type_name eq $self->name ) {
             my %map = reverse %{ $self->document->ns_map };
             return $map{$self->document->target_namespace} . ':' . $type;
         }
@@ -236,14 +246,14 @@ sub has_anonymous {
     my $complex = $self->document->complex_type;
     for my $type (keys %{$complex}) {
         my  $type_name = $complex->{$type}->node->parentNode->getAttribute('name');
-        if ( $type_name && $type_name eq $self->name ) {
+        if ( $type_name && $self->name && $type_name eq $self->name ) {
             my %map = reverse %{ $self->document->ns_map };
             return $map{$self->document->target_namespace} . ':' . $type;
         }
         $type_name ||= '';
     }
 
-    return;
+    return 'xs:string';
 }
 
 1;
@@ -256,7 +266,7 @@ W3C::SOAP::XSD::Document::Element - XML Schema Element
 
 =head1 VERSION
 
-This documentation refers to W3C::SOAP::XSD::Document::Element version 0.0.2.
+This documentation refers to W3C::SOAP::XSD::Document::Element version 0.0.3.
 
 
 =head1 SYNOPSIS
